#!/bin/sh
#
# Script for generating the autotools configure script
#

# Automake 1.14+ warns if the subdir-objects option is not used.
# This check removes the option for older versions, some of which cause a build failure.
v=$(automake --version | { read w1 w2 w3 version w5 && echo ${version}; })
case $v in
    1.1[4-9]*|2*) ;; # versions 1.14 and higher require no change to configure.ac
    *) perl -p -i.copy -e 's/(AM_INIT_AUTOMAKE\(\[.*?)\s?subdir-objects(.*)/$1$2/' configure.ac ;;
esac
libtoolize --ltdl --force --copy --automake  # version >= 1.5.2
aclocal -I Config                     # version >= 1.11 for Python 3, 1.8.2 for Python 2.6-2.7
autoheader                            # version >= 2.59
automake -a -c                        # version >= 1.11 for Python 3, 1.8.2 for Python 2.6-2.7
autoconf                              # version >= 2.59
[ -f configure.ac.copy ] && mv configure.ac.copy configure.ac

# generate bzr revision header file
if bzr version-info
then 
    bzr version-info --custom --template="static int s_bzr_revision={revno};\n" > bzrversion.h
    echo "bzr version header generated" 
elif [ -e bzrversion.h ]
then echo "using existing bzrversion.h"
else 
    echo "static int s_bzr_revision=-1;" > bzrversion.h
    echo "writing dummy bzr revision -1 to header file"
fi
